#include "precomp.h"
#include "scope_guard.h"
#include "string_util.h"
#include "shell-icon-provider.h"

#define INVALID_INDEX (0xDEADBEEF)

QString ShellIconProvider::providerPrefix = "shell";
QString ShellIconProvider::largeIconPrefix = "large";
QString ShellIconProvider::smallIconPrefix = "small";

QQuickImageProvider::ImageType ShellIconProvider::imageType() const
{
    return QQmlImageProviderBase::Pixmap;
}

QPixmap ShellIconProvider::getIconFromShellItemImage(const QString& path, const QSize &requestedSize)
{
    HRESULT     hr;

    // create IShellItem for this path
    IShellItem *item = nullptr;
    auto wcharPath = WcharArray::fromQString(path);
    hr = SHCreateItemFromParsingName(wcharPath->data, nullptr, IID_IShellItem, (void**)&item);
    if (FAILED(hr)) return QPixmap();
    ON_SCOPE_EXIT([&]() {item->Release(); });
    
    // get image factory
    IShellItemImageFactory *imageFactory = nullptr;
    hr = item->QueryInterface(IID_IShellItemImageFactory, (void**)&imageFactory);
    if (FAILED(hr))
        return QPixmap();
    ON_SCOPE_EXIT([&]() {imageFactory->Release(); });

    // get image for this item
    SIZE imageSize = { 0 };
    if (requestedSize.isValid()) {
        imageSize.cx = requestedSize.width();
        imageSize.cy = requestedSize.height();
    } else {
        imageSize.cx = imageSize.cy = 256;
    }

    HBITMAP bitmap = nullptr;
    hr = imageFactory->GetImage(imageSize, SIIGBF_BIGGERSIZEOK, &bitmap);
    if (FAILED(hr)) return QPixmap();
    ON_SCOPE_EXIT([&]() {DeleteObject(bitmap); });
    
    // convert to QPixmap
    QPixmap image = QtWin::fromHBITMAP(bitmap, QtWin::HBitmapPremultipliedAlpha);
    if (image.isNull())
        return QPixmap();

    // get the right size
    if (requestedSize.isValid() && image.size() != requestedSize)
        return image.scaled(requestedSize);
    return image;
}

QPixmap ShellIconProvider::requestPixmap(const QString &id, QSize *size, const QSize &requestedSize)
{
    QPixmap image;

    // may this is a new thread, we should initialize COM for it first at all
    CoInitialize(nullptr);

    // parse key
    auto parts = id.split("/");
    Q_ASSERT(parts.size() == 2);

    QString path = QString::fromLocal8Bit(QByteArray::fromBase64(parts[1].toLocal8Bit(), QByteArray::Base64UrlEncoding));
    if (path.isEmpty())
        return QPixmap();
    Q_ASSERT(QString::compare(parts[0], "image") == 0);

    // get shell image
    image = getIconFromShellItemImage(path, requestedSize);
    if (!image.isNull()) {
        *size = image.size();
        return image;
    }

    return QPixmap();
}

QString ShellIconProvider::getImageURIByPath(const QString& path)
{
    // generate URI
    auto uri = QString("image://%1/image/%3").arg(getPrefix()).arg(QString(path.toLocal8Bit().toBase64(QByteArray::Base64UrlEncoding)));
    return uri;
}

QString ShellIconProvider::getPrefix()
{
    return providerPrefix;
}
